@@ -1,11 +1,12 @@
+#![allow(clippy::too_many_arguments)]
+
 use std::{
     collections::{HashMap, HashSet},
     env,
-    path::MAIN_SEPARATOR,
 };
 
 use crate::{
-    command::Command,
+    command::{Command, SymbolParam},
     compgen::CompColor,
     param::{ChoiceData, FlagOptionParam, ParamData, PositionalParam},
     utils::run_param_fns,
@@ -22,6 +23,7 @@ pub(crate) struct Matcher<'a, 'b> {
     args: &'b [String],
     flag_option_args: Vec<Vec<FlagOptionArg<'a, 'b>>>,
     positional_args: Vec<&'b str>,
+    symbol_args: Vec<SymbolArg<'a, 'b>>,
     dashes: Option<usize>,
     arg_comp: ArgComp,
     choice_fns: HashSet<&'a str>,
@@ -32,6 +34,7 @@ pub(crate) struct Matcher<'a, 'b> {
 }
 
 type FlagOptionArg<'a, 'b> = (&'b str, Vec<&'b str>, Option<&'a str>); // key, values, param_name
+type SymbolArg<'a, 'b> = (&'b str, &'a SymbolParam);
 type LevelCommand<'a, 'b> = (&'b str, &'a Command, usize); // name, command, arg_index
 
 #[derive(Debug, Clone, PartialEq, Eq)]
@@ -40,6 +43,7 @@ pub(crate) enum ArgComp {
     FlagOrOptionCombine(String),
     CommandOrPositional,
     OptionValue(String, usize),
+    Symbol(char),
     Any,
 }
 
@@ -65,12 +69,13 @@ pub(crate) enum MatchError {
 
 impl<'a, 'b> Matcher<'a, 'b> {
     pub(crate) fn new(root: &'a Command, args: &'b [String], compgen: bool) -> Self {
-        let combine_shorts = root.metadata.contains_key("combine-shorts");
+        let combine_shorts = root.has_metadata("combine-shorts");
         let mut cmds: Vec<LevelCommand> = vec![(args[0].as_str(), root, 0)];
         let mut cmd_level = 0;
         let mut arg_index = 1;
         let mut flag_option_args = vec![vec![]];
         let mut positional_args = vec![];
+        let mut symbol_args = vec![];
         let mut dashes = None;
         let mut split_last_arg_at = None;
         let mut arg_comp = ArgComp::Any;
@@ -82,7 +87,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
         } else {
             let mut is_rest_args_positional = false; // option(e.g. -f --foo) will be treated as positional arg
             if let Some(arg) = args.last() {
-                if arg.starts_with('-') {
+                if arg.starts_with(|c| root.flag_option_signs().contains(c)) {
                     arg_comp = ArgComp::FlagOrOption;
                 } else if !arg.is_empty() {
                     arg_comp = ArgComp::CommandOrPositional;
@@ -90,6 +95,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
             }
             while arg_index < args_len {
                 let cmd = cmds[cmd_level].1;
+                let signs = cmd.flag_option_signs();
                 let arg = args[arg_index].as_str();
                 let is_last_arg = arg_index == args_len - 1;
                 last_flag_option = None;
@@ -117,16 +123,16 @@ impl<'a, 'b> Matcher<'a, 'b> {
                         &mut is_rest_args_positional,
                         cmd,
                     );
-                } else if arg.starts_with('-') {
+                } else if arg.starts_with(|c| signs.contains(c)) {
                     if let Some((k, v)) = arg.split_once('=') {
                         if let Some(param) = cmd.find_flag_option(k) {
                             add_param_choice_fn(&mut choice_fns, param);
                             if is_last_arg {
-                                arg_comp = ArgComp::OptionValue(param.name().to_string(), 0);
+                                arg_comp = ArgComp::OptionValue(param.var_name().to_string(), 0);
                                 split_last_arg_at = Some(k.len() + 1);
                             }
-                            flag_option_args[cmd_level].push((k, vec![v], Some(param.name())));
-                            last_flag_option = Some(param.name());
+                            flag_option_args[cmd_level].push((k, vec![v], Some(param.var_name())));
+                            last_flag_option = Some(param.var_name());
                         } else if let Some((param, prefix)) = cmd.find_prefixed_option(arg) {
                             add_param_choice_fn(&mut choice_fns, param);
                             match_prefix_option(
@@ -138,7 +144,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                                 &mut split_last_arg_at,
                                 &prefix,
                             );
-                            last_flag_option = Some(param.name());
+                            last_flag_option = Some(param.var_name());
                         } else {
                             flag_option_args[cmd_level].push((k, vec![v], None));
                         }
@@ -152,8 +158,9 @@ impl<'a, 'b> Matcher<'a, 'b> {
                             &mut arg_comp,
                             &mut split_last_arg_at,
                             combine_shorts,
+                            &signs,
                         );
-                        last_flag_option = Some(param.name());
+                        last_flag_option = Some(param.var_name());
                     } else if let Some((param, prefix)) = cmd.find_prefixed_option(arg) {
                         add_param_choice_fn(&mut choice_fns, param);
                         match_prefix_option(
@@ -165,7 +172,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                             &mut split_last_arg_at,
                             &prefix,
                         );
-                        last_flag_option = Some(param.name());
+                        last_flag_option = Some(param.var_name());
                     } else if let Some(subcmd) = find_subcommand(cmd, arg, &positional_args)
                         .and_then(|v| {
                             if is_last_arg && compgen {
@@ -219,8 +226,18 @@ impl<'a, 'b> Matcher<'a, 'b> {
                                 &mut arg_comp,
                                 &mut split_last_arg_at,
                                 combine_shorts,
+                                &signs,
                             );
-                            last_flag_option = Some(param.name());
+                            last_flag_option = Some(param.var_name());
+                        }
+                    } else if let Some((ch, symbol_param)) = find_symbol(cmd, arg) {
+                        if let Some(choice_fn) = &symbol_param.1 {
+                            choice_fns.insert(choice_fn);
+                        }
+                        symbol_args.push((&arg[1..], symbol_param));
+                        if is_last_arg {
+                            arg_comp = ArgComp::Symbol(ch);
+                            split_last_arg_at = Some(1);
                         }
                     } else {
                         flag_option_args[cmd_level].push((arg, vec![], None));
@@ -243,6 +260,15 @@ impl<'a, 'b> Matcher<'a, 'b> {
                         arg_index,
                         &mut is_rest_args_positional,
                     );
+                } else if let Some((ch, symbol_param)) = find_symbol(cmd, arg) {
+                    if let Some(choice_fn) = &symbol_param.1 {
+                        choice_fns.insert(choice_fn);
+                    }
+                    symbol_args.push((&arg[1..], symbol_param));
+                    if is_last_arg {
+                        arg_comp = ArgComp::Symbol(ch);
+                        split_last_arg_at = Some(1);
+                    }
                 } else {
                     add_positional_arg(
                         &mut positional_args,
@@ -273,6 +299,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
             args,
             flag_option_args,
             positional_args,
+            symbol_args,
             dashes,
             arg_comp,
             choice_fns,
@@ -398,13 +425,14 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 if let Some(param) = last_cmd
                     .flag_option_params
                     .iter()
-                    .find(|v| v.name() == name)
+                    .find(|v| v.var_name() == name)
                 {
                     comp_flag_option(param, *index)
                 } else {
                     vec![]
                 }
             }
+            ArgComp::Symbol(ch) => comp_symbol(last_cmd, *ch),
             ArgComp::Any => {
                 if self.positional_args.len() == 2 && self.positional_args[0] == "help" {
                     return comp_subcomands(last_cmd, false);
@@ -437,6 +465,11 @@ impl<'a, 'b> Matcher<'a, 'b> {
         let level = cmds_len - 1;
         let last_cmd = self.cmds[level].1;
         let cmd_arg_index = self.cmds[level].2;
+
+        for (arg, (name, _)) in self.symbol_args.iter() {
+            output.push(ArgcValue::Single(name.to_string(), arg.to_string()));
+        }
+
         for level in 0..cmds_len {
             let args = &self.flag_option_args[level];
             let cmd = self.cmds[level].1;
@@ -444,7 +477,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 let values: Vec<&[&str]> = args
                     .iter()
                     .filter_map(|(_, value, name)| {
-                        if let Some(true) = name.map(|v| param.name() == v) {
+                        if let Some(true) = name.map(|v| param.var_name() == v) {
                             Some(value.as_slice())
                         } else {
                             None
@@ -578,7 +611,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 .flag_option_params
                 .iter()
                 .filter(|v| v.required())
-                .map(|v| v.name())
+                .map(|v| v.var_name())
                 .collect();
             for (i, (key, _, name)) in args.iter().enumerate() {
                 match *name {
@@ -592,12 +625,12 @@ impl<'a, 'b> Matcher<'a, 'b> {
             if !missing_flag_options.is_empty() {
                 let missing_flag_options: Vec<String> = missing_flag_options
                     .iter()
-                    .filter_map(|v| cmd.find_flag_option(v).map(|v| v.render_name_values()))
+                    .filter_map(|v| cmd.find_flag_option(v).map(|v| v.render_name_notations()))
                     .collect();
                 missing_params.extend(missing_flag_options)
             }
             for (name, indexes) in flag_option_map {
-                if let Some(param) = cmd.flag_option_params.iter().find(|v| v.name() == name) {
+                if let Some(param) = cmd.flag_option_params.iter().find(|v| v.var_name() == name) {
                     let values_list: Vec<&[&str]> =
                         indexes.iter().map(|v| args[*v].1.as_slice()).collect();
                     if !param.multi_occurs() && values_list.len() > 1 {
@@ -613,7 +646,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                         } else if !param.validate_args_len(values.len()) {
                             return Some(MatchError::MismatchValues(
                                 level,
-                                param.render_name_values(),
+                                param.render_name_notations(),
                             ));
                         }
                         if let Some(choices) =
@@ -624,7 +657,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                                     return Some(MatchError::InvalidValue(
                                         level,
                                         value.to_string(),
-                                        param.render_first_value(),
+                                        param.render_first_notation(),
                                         choices.clone(),
                                     ));
                                 }
@@ -653,7 +686,6 @@ impl<'a, 'b> Matcher<'a, 'b> {
         let fns: Vec<&str> = self.choice_fns.iter().copied().collect();
         let mut envs = HashMap::new();
         envs.insert("ARGC_OS".into(), env::consts::OS.to_string());
-        envs.insert("ARGC_PATH_SEP".into(), MAIN_SEPARATOR.into());
         let outputs = run_param_fns(script_path, &fns, self.args, envs)?;
         for (i, output) in outputs.into_iter().enumerate() {
             let choices = output
@@ -805,7 +837,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
                 let (cmd, cmd_paths) = self.get_cmd_and_paths(*level);
                 let usage = cmd.render_usage(&cmd_paths);
                 format!(
-                    r###"error: invalid values for `{value}`
+                    r###"error: invalid value for `{value}`
 
 {usage}
 
@@ -846,18 +878,18 @@ impl<'a, 'b> Matcher<'a, 'b> {
             .collect();
         let last = self.args.last().map(|v| v.as_str()).unwrap_or_default();
         for param in cmd.flag_option_params.iter() {
-            let mut exist = args.contains(param.name());
+            let mut exist = args.contains(param.var_name());
             if !last.is_empty() && param.is_match(last) {
                 exist = false;
             }
             if !exist || param.multi_occurs() {
-                let describe = param.describe_head();
+                let describe = param.describe_oneline();
                 let kind = if param.is_flag() {
                     CompColor::of_flag()
                 } else {
                     CompColor::of_option()
                 };
-                for v in param.list_names() {
+                for v in param.list_option_names() {
                     output.push((v, describe.to_string(), param.prefixed().is_some(), kind))
                 }
             }
@@ -866,6 +898,7 @@ impl<'a, 'b> Matcher<'a, 'b> {
     }
 }
 
+/// (value, description, nospace, color)
 pub(crate) type CompItem = (String, String, bool, CompColor);
 
 fn find_subcommand<'a>(
@@ -882,6 +915,15 @@ fn find_subcommand<'a>(
     })
 }
 
+fn find_symbol<'a>(cmd: &'a Command, arg: &str) -> Option<(char, &'a SymbolParam)> {
+    for (ch, param) in cmd.symbols.iter() {
+        if arg.starts_with(*ch) {
+            return Some((*ch, param));
+        }
+    }
+    None
+}
+
 fn add_positional_arg<'a>(
     positional_args: &mut Vec<&'a str>,
     arg: &'a str,
@@ -897,14 +939,14 @@ fn add_positional_arg<'a>(
     }
 }
 
-fn take_value_args(args: &[String], start: usize, len: usize) -> Vec<&str> {
+fn take_value_args<'a>(args: &'a [String], start: usize, len: usize, signs: &str) -> Vec<&'a str> {
     let mut output = vec![];
     if len == 0 {
         return output;
     }
     let end = (start + len).min(args.len());
     for arg in args.iter().take(end).skip(start) {
-        if arg.starts_with('-') {
+        if arg.starts_with(|c| signs.contains(c)) {
             break;
         }
         output.push(arg.as_str());
@@ -936,7 +978,7 @@ fn match_combine_shorts<'a, 'b>(
             }
         }
         if let Some(param) = current_cmd.find_flag_option(&name) {
-            output.push((arg, vec![], Some(param.name())))
+            output.push((arg, vec![], Some(param.var_name())))
         } else {
             return None;
         }
@@ -953,43 +995,46 @@ fn match_flag_option<'a, 'b>(
     arg_comp: &mut ArgComp,
     split_last_arg_at: &mut Option<usize>,
     combine_shorts: bool,
+    signs: &str,
 ) {
     if param.terminated() {
         let value_args: Vec<&str> = args[*arg_index + 1..].iter().map(|v| v.as_str()).collect();
         let arg = &args[*arg_index];
         *arg_index += value_args.len();
         if !value_args.is_empty() {
-            *arg_comp =
-                ArgComp::OptionValue(param.name().to_string(), value_args.len().saturating_sub(1));
+            *arg_comp = ArgComp::OptionValue(
+                param.var_name().to_string(),
+                value_args.len().saturating_sub(1),
+            );
         }
-        flag_option_args.push((arg, value_args, Some(param.name())));
+        flag_option_args.push((arg, value_args, Some(param.var_name())));
     } else {
         let mut values_len = param.arg_value_names.len();
         if param.unlimited_args() {
             values_len = usize::MAX / 2;
         }
         let args_len = args.len();
-        let value_args = take_value_args(args, *arg_index + 1, values_len);
+        let value_args = take_value_args(args, *arg_index + 1, values_len, signs);
         let arg = &args[*arg_index];
         *arg_index += value_args.len();
         if *arg_index == args_len - 1 {
             if *arg_comp != ArgComp::FlagOrOption {
                 if param.is_option() && value_args.len() <= values_len {
                     *arg_comp = ArgComp::OptionValue(
-                        param.name().to_string(),
+                        param.var_name().to_string(),
                         value_args.len().saturating_sub(1),
                     );
                 }
             } else if let Some(prefix) = param.prefixed() {
                 if arg.starts_with(&prefix) {
-                    *arg_comp = ArgComp::OptionValue(param.name().to_string(), 0);
+                    *arg_comp = ArgComp::OptionValue(param.var_name().to_string(), 0);
                     *split_last_arg_at = Some(prefix.len());
                 }
             } else if combine_shorts && param.is_flag() && !(arg.len() > 2 && param.is_match(arg)) {
                 *arg_comp = ArgComp::FlagOrOptionCombine(arg.to_string());
             }
         }
-        flag_option_args.push((arg, value_args, Some(param.name())));
+        flag_option_args.push((arg, value_args, Some(param.var_name())));
     }
 }
 
@@ -1006,10 +1051,10 @@ fn match_prefix_option<'a, 'b>(
     let args_len = args.len();
     let arg = &args[*arg_index];
     if *arg_index == args_len - 1 {
-        *arg_comp = ArgComp::OptionValue(param.name().to_string(), 0);
+        *arg_comp = ArgComp::OptionValue(param.var_name().to_string(), 0);
         *split_last_arg_at = Some(prefix_len);
     }
-    flag_option_args.push((arg, vec![&arg[prefix_len..]], Some(param.name())));
+    flag_option_args.push((arg, vec![&arg[prefix_len..]], Some(param.var_name())));
 }
 
 fn match_command<'a, 'b>(
@@ -1057,13 +1102,16 @@ fn comp_subcomands(cmd: &Command, flag: bool) -> Vec<CompItem> {
     let mut output = vec![];
     let mut has_help_subcmd = false;
     let mut describe_help_subcmd = false;
+    let signs = cmd.flag_option_signs();
     for subcmd in cmd.subcommands.iter() {
-        let describe = subcmd.describe_head();
+        let describe = subcmd.describe_oneline();
         for (i, v) in subcmd.list_names().into_iter().enumerate() {
             if i > 0 && v.len() < 2 {
                 continue;
             }
-            if (flag && v.starts_with('-')) || (!flag && !v.starts_with('-')) {
+            if (flag && v.starts_with(|c| signs.contains(c)))
+                || (!flag && !v.starts_with(|c| signs.contains(c)))
+            {
                 if !flag {
                     has_help_subcmd = true;
                 }
@@ -1090,17 +1138,42 @@ fn comp_subcomands(cmd: &Command, flag: bool) -> Vec<CompItem> {
     output
 }
 
+fn comp_symbol(cmd: &Command, ch: char) -> Vec<CompItem> {
+    if let Some((name, choices_fn)) = cmd.symbols.get(&ch) {
+        match choices_fn {
+            Some(choices_fn) => {
+                vec![(
+                    format!("__argc_fn={}", choices_fn),
+                    String::new(),
+                    false,
+                    CompColor::of_value(),
+                )]
+            }
+            None => {
+                vec![(
+                    format!("__argc_value={}", name),
+                    String::new(),
+                    false,
+                    CompColor::of_value(),
+                )]
+            }
+        }
+    } else {
+        vec![]
+    }
+}
+
 fn comp_flag_option(param: &FlagOptionParam, index: usize) -> Vec<CompItem> {
     let value_name = param
         .arg_value_names
         .get(index)
         .map(|v| v.as_str())
         .unwrap_or_else(|| param.arg_value_names.last().unwrap());
-    comp_param(param.describe_head(), value_name, &param.data)
+    comp_param(param.describe_oneline(), value_name, &param.data)
 }
 
 fn comp_positional(param: &PositionalParam) -> Vec<CompItem> {
-    comp_param(param.describe_head(), &param.arg_value_name, &param.data)
+    comp_param(param.describe_oneline(), &param.arg_value_name, &param.data)
 }
 
 fn comp_param(describe: &str, value_name: &str, data: &ParamData) -> Vec<CompItem> {
