@@ -305,3 +305,52 @@ cmdb() {
         ]
     );
 }
+
+#[test]
+fn symbol() {
+    let script = r###"
+# @meta symbol +toolchain
+# @option --oa
+"###;
+    snapshot_multi!(script, [vec!["prog", "+nightly"]]);
+}
+
+#[test]
+fn plus_sign() {
+    let script = r###"
+# @flag +a
+# @option +fb
+# @option +c +fc*
+# @option +d -fd*
+"###;
+    snapshot_multi!(
+        script,
+        [
+            vec!["prog", "+a", "+fb", "fb", "+c", "fc1", "+fc", "fc2"],
+            vec!["prog", "+d", "fd1", "-fd", "fd2"],
+        ]
+    );
+}
+
+#[test]
+fn notation_modifier() {
+    let script = r###"
+# @option --oa <VALUE*>           multi values, zero or more
+# @option --ob <VALUE+>           multi values, one or more
+# @option --oc <VALUE?>           zero or one
+"###;
+    snapshot_multi!(
+        script,
+        [
+            vec!["prog", "--oa"],
+            vec!["prog", "--oa", "v1"],
+            vec!["prog", "--oa", "v1", "v2"],
+            vec!["prog", "--ob"],
+            vec!["prog", "--ob", "v1"],
+            vec!["prog", "--ob", "v1", "v2"],
+            vec!["prog", "--oc"],
+            vec!["prog", "--oc", "v1"],
+            vec!["prog", "--oc", "v1", "v2"],
+        ]
+    );
+}
